-- scenariusz: tabela gwna - indeks rowstore, tabela historyczna - indeks columnstore

USE SQL2016Demo;
GO

-- BR: 1m2s | 58s | 59s
SELECT TOP 10000000
    ISNULL(p.ProductID + (a.number * 1000) + (b.number * 100000000),0) AS ProductID,
    p.Name + CONVERT(VARCHAR, (a.number * 1000)) AS Name,
    p.ProductNumber + '-' + CONVERT(VARCHAR, (a.number * 1000)) AS ProductNumber,
    p.MakeFlag,
    p.FinishedGoodsFlag,
    p.Color,
    p.SafetyStockLevel,
    p.ReorderPoint,
    p.StandardCost,
    p.ListPrice,
    p.Size,
    p.SizeUnitMeasureCode,
    p.WeightUnitMeasureCode,
    p.Weight,
    p.DaysToManufacture,
    p.ProductLine,
    p.Class,
    p.Style,
    p.ProductSubcategoryID,
    p.ProductModelID,
    p.SellStartDate,
    p.SellEndDate,
    p.DiscontinuedDate,
    cast('' as char(100)) as DataFiller 
INTO SQL2016Demo.dbo.ProductW2
FROM AdventureWorks2014.Production.Product AS p
CROSS JOIN master..spt_values AS a
CROSS JOIN master..spt_values AS b
WHERE a.type = 'p' and b.type= 'p' and b.number between 1 and 10

-- Data - 2975 MB 
exec sp_spaceused 'dbo.ProductW2'

GO
-- BR: 4m23s | 4m5s | 3m51s
SELECT TOP 1000000 ProductID INTO dbo.ProductW2_SET1_1MLN FROM dbo.ProductW2
ORDER BY NEWID()

SELECT TOP 1000000 ProductID INTO dbo.ProductW2_SET2_1MLN FROM dbo.ProductW2
ORDER BY NEWID()

SELECT TOP 1000000 ProductID INTO dbo.ProductW2_SET3_1MLN FROM dbo.ProductW2
ORDER BY NEWID()

SELECT TOP 1000000 ProductID INTO dbo.ProductW2_SET4_1MLN FROM dbo.ProductW2
ORDER BY NEWID()






-- AV: 07m 08s | BR: 9m37s | 8m28s | 7m46s
ALTER TABLE dbo.ProductW2 ADD
	CONSTRAINT PK_Product PRIMARY KEY CLUSTERED (
		ProductID
	) WITH (
		STATISTICS_NORECOMPUTE	= OFF,
		IGNORE_DUP_KEY			= OFF
	) ON [PRIMARY]

-- Data - 2989 MB 
exec sp_spaceused 'dbo.ProductW2'

-- AV: 1s | BR: 28s | 18s | 48s
BEGIN TRAN
ALTER TABLE dbo.ProductW2
ADD PERIOD FOR SYSTEM_TIME (Valid_From, Valid_Till),
Valid_From datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT GETUTCDATE(),
Valid_Till datetime2 GENERATED ALWAYS AS ROW END NOT NULL DEFAULT '9999-12-31 23:59:59.9999999';
ALTER TABLE dbo.ProductW2 SET (SYSTEM_VERSIONING = ON (HISTORY_TABLE = dbo.ProductW2_History, DATA_CONSISTENCY_CHECK = OFF))
COMMIT TRAN


-- BR: <1s
DROP INDEX ix_ProductW2_History
	ON dbo.ProductW2_History
	WITH (
		ONLINE = OFF
	)
GO

CREATE CLUSTERED COLUMNSTORE INDEX cci_ProductW2_History
	ON dbo.ProductW2_History
	WITH (
		DROP_EXISTING = OFF
	) ON [PRIMARY]
GO


-- AV: 309 seconds | BR: 5m24s | 5m38s | 5m36s
UPDATE  dbo.ProductW2
SET Weight = RAND()
WHERE ProductID in ( SELECT ProductID FROM  dbo.ProductW2_SET1_1MLN )
 
-- AV: 377 seconds | BR: 7m37s | 8m42s | 6m57s
DELETE FROM  dbo.ProductW2
WHERE ProductID in ( SELECT ProductID FROM  dbo.ProductW2_SET2_1MLN )

-- AV: 513 seconds | BR: 8m02s | 8m29s | 7m26s
UPDATE  dbo.ProductW2
SET Weight = RAND()
WHERE ProductID in ( SELECT ProductID FROM  dbo.ProductW2_SET3_1MLN )

-- AV: 466 seconds | BR: 8m05s | 8m22s | 7m48s
DELETE FROM  dbo.ProductW2
WHERE ProductID in ( SELECT ProductID FROM  dbo.ProductW2_SET4_1MLN )

-- overall: 1665 seconds

-- BR: 4m35s | 4m22s | 4m44s | 4m44s
ALTER INDEX ALL ON [dbo].[ProductW2] REBUILD 
ALTER INDEX ALL ON [dbo].[ProductW2_History] REBUILD 

-- data 100 MB | BR: 90.8 MB | 89 MB | 91 MB
exec sp_spaceused 'dbo.ProductW2_History'

-- data 2 556 MB | BR: 2560 MB | 2560 MB | 2560 MB
exec sp_spaceused 'dbo.ProductW2'


-- BR: 1s
select distinct [Valid_From] from dbo.ProductW2_History

--2015-06-26 13:01:12.1770189
--2015-06-26 12:47:08.0070000
--2015-06-26 12:47:26.7097046

-- BR
--2015-11-17 08:25:37.1776749
--2015-11-17 07:54:05.9866667 <-- wybieramy rodkowy czas
--2015-11-17 07:56:25.8960224

--2015-11-17 14:04:54.9502030
--2015-11-17 13:29:38.7464878 <-- wybieramy rodkowy czas
--2015-11-17 13:28:39.8866667

--2015-11-17 15:24:03.4052181
--2015-11-17 15:22:52.8566667 <-- wybieramy rodkowy czas
--2015-11-17 15:47:22.9057708

SET STATISTICS IO, TIME ON

--  SQL Server Execution Times:    CPU time = 32 ms,  elapsed time = 33 ms.
SELECT SUM(StandardCost) from dbo.ProductW2
WHERE ProductID BETWEEN 100000000 AND 100200000


--    CPU time = 47 ms,  elapsed time = 36 ms.
SELECT SUM(StandardCost) FROM dbo.ProductW2
FOR SYSTEM_TIME AS OF '2015-11-17 15:22:52.8566667' 
WHERE ProductID BETWEEN 100000000 AND 100200000


-- 68 911 ms
SELECT SUM(StandardCost) FROM dbo.ProductW2
WHERE Weight > 0.5


--  CPU time = 6092 ms,  elapsed time = 70657 ms.
SELECT SUM(StandardCost) FROM dbo.ProductW2
FOR SYSTEM_TIME AS OF '2015-11-17 15:22:52.8566667' 
WHERE Weight > 0.5

/*
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW2'. Scan count 1, logical reads 3192, physical reads 2, read-ahead reads 3055, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 16 ms,  elapsed time = 539 ms.

(1 row(s) affected)
Table 'ProductW2_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 3701, lob physical reads 10, lob read-ahead reads 14073.
Table 'ProductW2_History'. Segment reads 4, segment skipped 1.
Table 'ProductW2'. Scan count 1, logical reads 3192, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 31 ms,  elapsed time = 438 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW2'. Scan count 3, logical reads 320400, physical reads 10, read-ahead reads 302158, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 1281 ms,  elapsed time = 51834 ms.

(1 row(s) affected)
Table 'ProductW2_History'. Scan count 2, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 143, lob physical reads 12, lob read-ahead reads 12.
Table 'ProductW2_History'. Segment reads 5, segment skipped 0.
Table 'ProductW2'. Scan count 3, logical reads 320304, physical reads 4258, read-ahead reads 191893, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
Table 'Worktable'. Scan count 0, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 1500 ms,  elapsed time = 37117 ms.
*/

/*
Druga prba

SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW2'. Scan count 1, logical reads 3196, physical reads 1, read-ahead reads 2780, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 0 ms,  elapsed time = 779 ms.

(1 row(s) affected)
Table 'ProductW2_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 3198, lob physical reads 1, lob read-ahead reads 6.
Table 'ProductW2_History'. Segment reads 3, segment skipped 1.
Table 'ProductW2'. Scan count 1, logical reads 3196, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 16 ms,  elapsed time = 89 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW2'. Scan count 3, logical reads 320415, physical reads 1, read-ahead reads 302167, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 1078 ms,  elapsed time = 50999 ms.

(1 row(s) affected)
Table 'ProductW2_History'. Scan count 2, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 115, lob physical reads 4, lob read-ahead reads 18.
Table 'ProductW2_History'. Segment reads 4, segment skipped 0.
Table 'ProductW2'. Scan count 3, logical reads 320439, physical reads 1452, read-ahead reads 258956, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
Table 'Worktable'. Scan count 0, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 1390 ms,  elapsed time = 48166 ms.
*/

/*
Trzecia prba
SQL Server parse and compile time: 
   CPU time = 2561 ms, elapsed time = 30813 ms.

 SQL Server Execution Times:
   CPU time = 0 ms,  elapsed time = 0 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW2'. Scan count 1, logical reads 3179, physical reads 0, read-ahead reads 3003, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 31 ms,  elapsed time = 674 ms.

(1 row(s) affected)
Table 'ProductW2_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 3476, lob physical reads 3, lob read-ahead reads 13265.
Table 'ProductW2_History'. Segment reads 4, segment skipped 1.
Table 'ProductW2'. Scan count 1, logical reads 3179, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 63 ms,  elapsed time = 324 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW2'. Scan count 3, logical reads 320456, physical reads 144, read-ahead reads 291408, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 1609 ms,  elapsed time = 52771 ms.

(1 row(s) affected)
Table 'ProductW2_History'. Scan count 2, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 148, lob physical reads 9, lob read-ahead reads 18.
Table 'ProductW2_History'. Segment reads 5, segment skipped 0.
Table 'ProductW2'. Scan count 3, logical reads 320416, physical reads 4317, read-ahead reads 212906, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.
Table 'Worktable'. Scan count 0, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 0, lob physical reads 0, lob read-ahead reads 0.

 SQL Server Execution Times:
   CPU time = 1750 ms,  elapsed time = 41629 ms.
*/