-- scenariusz 3: tabela gwna - columnstore index, tabela historyczna: columnstore index

USE SQL2016Demo;
GO

-- BR: 1m1s | 1m3s | 1m3s
SELECT TOP 10000000
    ISNULL(p.ProductID + (a.number * 1000) + (b.number * 100000000),0) AS ProductID,
    p.Name + CONVERT(VARCHAR, (a.number * 1000)) AS Name,
    p.ProductNumber + '-' + CONVERT(VARCHAR, (a.number * 1000)) AS ProductNumber,
    p.MakeFlag,
    p.FinishedGoodsFlag,
    p.Color,
    p.SafetyStockLevel,
    p.ReorderPoint,
    p.StandardCost,
    p.ListPrice,
    p.Size,
    p.SizeUnitMeasureCode,
    p.WeightUnitMeasureCode,
    p.Weight,
    p.DaysToManufacture,
    p.ProductLine,
    p.Class,
    p.Style,
    p.ProductSubcategoryID,
    p.ProductModelID,
    p.SellStartDate,
    p.SellEndDate,
    p.DiscontinuedDate,
    cast('' as char(100)) as DataFiller 
INTO SQL2016Demo.dbo.ProductW3
FROM AdventureWorks2014.Production.Product AS p
CROSS JOIN master..spt_values AS a
CROSS JOIN master..spt_values AS b
WHERE a.type = 'p' and b.type= 'p' and b.number between 1 and 10

-- Data - 2975 MB 
exec sp_spaceused 'dbo.ProductW3'
GO

-- BR: 3m58s | 4m16s | 4m3s
SELECT TOP 1000000 ProductID INTO dbo.ProductW3_SET1_1MLN FROM dbo.ProductW3
ORDER BY NEWID()

SELECT TOP 1000000 ProductID INTO dbo.ProductW3_SET2_1MLN FROM dbo.ProductW3
ORDER BY NEWID()

SELECT TOP 1000000 ProductID INTO dbo.ProductW3_SET3_1MLN FROM dbo.ProductW3
ORDER BY NEWID()

SELECT TOP 1000000 ProductID INTO dbo.ProductW3_SET4_1MLN FROM dbo.ProductW3
ORDER BY NEWID()


-- AV: 128 seconds | BR: 2m12s | 2m09s | 2m19s
CREATE CLUSTERED COLUMNSTORE INDEX CCI_ProductW3
	ON dbo.ProductW3
	WITH (
		DROP_EXISTING = OFF
	)


-- AV: 18 seconds | BR: 31s | 33s | 34s
ALTER TABLE dbo.ProductW3 ADD
	CONSTRAINT PK_ProductW3 PRIMARY KEY NONCLUSTERED (
		ProductID
	) WITH (
		STATISTICS_NORECOMPUTE	= OFF,
		IGNORE_DUP_KEY			= OFF
	) ON [PRIMARY]



-- Data - 340 MB | BR: 312 MB | 527MB (reserved) / 310 MB
exec sp_spaceused 'dbo.ProductW3'

-- 1s
BEGIN TRAN
ALTER TABLE dbo.ProductW3
ADD PERIOD FOR SYSTEM_TIME (Valid_From, Valid_Till),
Valid_From datetime2 GENERATED ALWAYS AS ROW START NOT NULL DEFAULT GETUTCDATE(),
Valid_Till datetime2 GENERATED ALWAYS AS ROW END NOT NULL DEFAULT '9999-12-31 23:59:59.9999999';
ALTER TABLE dbo.ProductW3 SET (SYSTEM_VERSIONING = ON (HISTORY_TABLE = dbo.ProductW3_History, DATA_CONSISTENCY_CHECK = OFF))
COMMIT TRAN

-- BR: <1s
DROP INDEX ix_ProductW3_History
	ON dbo.ProductW3_History
	WITH (
		ONLINE = OFF
	)
GO

CREATE CLUSTERED COLUMNSTORE INDEX cci_ProductW3_History
	ON dbo.ProductW3_History
	WITH (
		DROP_EXISTING = OFF
	) ON [PRIMARY]
GO


-- AV: 255 seconds | BR: 2m39s | 2m45s | 2m22s
UPDATE  dbo.ProductW3
SET Weight = RAND()
WHERE ProductID in ( SELECT ProductID FROM  dbo.ProductW3_SET1_1MLN )
 
-- AV: 29 seconds | BR: 44s | 1m31s | 1m6s
DELETE FROM  dbo.ProductW3
WHERE ProductID in ( SELECT ProductID FROM  dbo.ProductW3_SET2_1MLN )

-- AV: 243 seconds | BR: 2m48s | 3m42s | 3m26s
UPDATE  dbo.ProductW3
SET Weight = RAND()
WHERE ProductID in ( SELECT ProductID FROM  dbo.ProductW3_SET3_1MLN )

-- 46 seconds | BR: prerwano po 44m33s, LiveQuery - Czeka na TableSpool 
-- BR po przebudowie indeksw: 31s
-- Drugie podejscie: 1m48s
-- Trzecie podejcie: 1m31s
DELETE FROM  dbo.ProductW3
WHERE ProductID in ( SELECT ProductID FROM  dbo.ProductW3_SET4_1MLN )

-- overall: 573 seconds

-- (przed DELETE SET4: 20s, po DELETE SET4: 2m2s) | 3m34s | 2m25s
ALTER INDEX ALL ON dbo.ProductW3 REBUILD 
ALTER INDEX ALL ON dbo.ProductW3_History REBUILD 

-- data 98 MB | BR: 100 MB | 92 MB | 101 MB
exec sp_spaceused 'dbo.ProductW3_History'

-- data 223 MB | 376 MB | 360 MB | 330 MB
exec sp_spaceused 'dbo.ProductW3'



SELECT DISTINCT Valid_From FROM dbo.ProductW3_History

--2015-06-26 13:36:16.2316136
--2015-06-26 13:36:09.9500000
--2015-06-26 13:44:55.5553411


-- BR
--2015-11-17 09:56:17.0389628
--2015-11-17 09:41:44.3500000 <-- wybieramy rodkowy czas
--2015-11-17 09:42:30.9601312

--2015-11-17 11:42:15.2286270
--2015-11-17 11:37:03.8531828 <-- wybieramy rodkowy czas
--2015-11-17 11:35:48.6600000

--2015-11-17 12:09:38.2600000
--2015-11-17 12:09:49.8696320 <-- wybieramy rodkowy czas
--2015-11-17 12:16:36.7142672

SET STATISTICS IO, TIME ON

--  CPU time = 16 ms,  elapsed time = 10 ms.
SELECT SUM(StandardCost) FROM dbo.ProductW3
WHERE ProductID BETWEEN 100000000 AND 100200000

--    CPU time = 16 ms,  elapsed time = 13 ms.
SELECT SUM(StandardCost) FROM dbo.ProductW3
FOR SYSTEM_TIME AS OF '2015-11-17 12:09:49.8696320' 
WHERE ProductID BETWEEN 100000000 AND 100200000


-- CPU time = 0 ms,  elapsed time = 3 ms.
SELECT SUM(StandardCost) FROM dbo.ProductW3
WHERE Weight > 0.5


--   CPU time = 16 ms,  elapsed time = 7 ms.
SELECT SUM(StandardCost) FROM dbo.ProductW3
FOR SYSTEM_TIME AS OF '2015-11-17 12:09:49.8696320' 
WHERE Weight > 0.5

/*
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 6515, lob physical reads 1, lob read-ahead reads 25592.
Table 'ProductW3'. Segment reads 8, segment skipped 2.

 SQL Server Execution Times:
   CPU time = 16 ms,  elapsed time = 650 ms.

(1 row(s) affected)
Table 'ProductW3_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 3892, lob physical reads 1, lob read-ahead reads 0.
Table 'ProductW3_History'. Segment reads 4, segment skipped 0.
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 6432, lob physical reads 0, lob read-ahead reads 0.
Table 'ProductW3'. Segment reads 6, segment skipped 4.

 SQL Server Execution Times:
   CPU time = 16 ms,  elapsed time = 71 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 146, lob physical reads 0, lob read-ahead reads 0.
Table 'ProductW3'. Segment reads 10, segment skipped 0.

 SQL Server Execution Times:
   CPU time = 15 ms,  elapsed time = 25 ms.

(1 row(s) affected)
Table 'ProductW3_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 95, lob physical reads 0, lob read-ahead reads 0.
Table 'ProductW3_History'. Segment reads 4, segment skipped 0.
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 210, lob physical reads 0, lob read-ahead reads 0.
Table 'ProductW3'. Segment reads 8, segment skipped 2.

 SQL Server Execution Times:
   CPU time = 16 ms,  elapsed time = 17 ms.

*/

/* druga seria:
SQL Server parse and compile time: 
   CPU time = 4452 ms, elapsed time = 6677 ms.

 SQL Server Execution Times:
   CPU time = 0 ms,  elapsed time = 0 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 3937, lob physical reads 1, lob read-ahead reads 15411.
Table 'ProductW3'. Segment reads 4, segment skipped 5.

 SQL Server Execution Times:
   CPU time = 16 ms,  elapsed time = 236 ms.

(1 row(s) affected)
Table 'ProductW3_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 3685, lob physical reads 1, lob read-ahead reads 5601.
Table 'ProductW3_History'. Segment reads 4, segment skipped 1.
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 3982, lob physical reads 0, lob read-ahead reads 0.
Table 'ProductW3'. Segment reads 4, segment skipped 5.

 SQL Server Execution Times:
   CPU time = 15 ms,  elapsed time = 176 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 97, lob physical reads 4, lob read-ahead reads 0.
Table 'ProductW3'. Segment reads 9, segment skipped 0.

 SQL Server Execution Times:
   CPU time = 0 ms,  elapsed time = 112 ms.

(1 row(s) affected)
Table 'ProductW3_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 119, lob physical reads 1, lob read-ahead reads 0.
Table 'ProductW3_History'. Segment reads 5, segment skipped 0.
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 192, lob physical reads 0, lob read-ahead reads 0.
Table 'ProductW3'. Segment reads 9, segment skipped 0.

 SQL Server Execution Times:
   CPU time = 0 ms,  elapsed time = 64 ms.

*/

/*
Trzecia seria

SQL Server parse and compile time: 
   CPU time = 5140 ms, elapsed time = 8747 ms.

 SQL Server Execution Times:
   CPU time = 0 ms,  elapsed time = 0 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 4869, lob physical reads 1, lob read-ahead reads 19009.
Table 'ProductW3'. Segment reads 5, segment skipped 4.

 SQL Server Execution Times:
   CPU time = 15 ms,  elapsed time = 477 ms.

(1 row(s) affected)
Table 'ProductW3_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 3923, lob physical reads 1, lob read-ahead reads 6108.
Table 'ProductW3_History'. Segment reads 5, segment skipped 0.
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 4925, lob physical reads 0, lob read-ahead reads 19009.
Table 'ProductW3'. Segment reads 5, segment skipped 4.

 SQL Server Execution Times:
   CPU time = 32 ms,  elapsed time = 542 ms.
SQL Server parse and compile time: 
   CPU time = 0 ms, elapsed time = 0 ms.

(1 row(s) affected)
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 99, lob physical reads 1, lob read-ahead reads 0.
Table 'ProductW3'. Segment reads 9, segment skipped 0.

 SQL Server Execution Times:
   CPU time = 0 ms,  elapsed time = 52 ms.

(1 row(s) affected)
Table 'ProductW3_History'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 121, lob physical reads 3, lob read-ahead reads 0.
Table 'ProductW3_History'. Segment reads 5, segment skipped 0.
Table 'ProductW3'. Scan count 1, logical reads 0, physical reads 0, read-ahead reads 0, lob logical reads 196, lob physical reads 0, lob read-ahead reads 0.
Table 'ProductW3'. Segment reads 9, segment skipped 0.

 SQL Server Execution Times:
   CPU time = 15 ms,  elapsed time = 102 ms.
*/