USE SQLDayLite2015;
GO

-- Przygotowanie tabeli, ktr bdziemy nka a do koca prezentacji
CREATE TABLE dbo.Product (
	ID			INT IDENTITY CONSTRAINT PK_Product PRIMARY KEY,
	Name		VARCHAR(15) NOT NULL,
	Price		SMALLMONEY
)
GO


-- Od dzi chcemy mie histori danych i wykorzysta do tego temporal tables
-- Opcja 1) Startujemy z now tabel, dla odrnienia niech ma sufiks TT:
-- Uwaga - tabela musi mie klucz gwny

CREATE TABLE dbo.ProductTT (
	ID				INT IDENTITY CONSTRAINT PK_ProductTT PRIMARY KEY,
	Name			VARCHAR(15) NOT NULL,
	Price			SMALLMONEY,

	PERIOD FOR SYSTEM_TIME (ValidFrom, ValidTo),

	ValidFrom		DATETIME2(0) GENERATED ALWAYS AS ROW START,
	ValidTo			DATETIME2(0) GENERATED ALWAYS AS ROW END
)
GO

-- Wczamy wersjonowanie i ogldamy co si dzieje w Object Explorer SSMS
-- (w tym zwracamy uwag na indeksy)
ALTER TABLE dbo.ProductTT
SET (
	SYSTEM_VERSIONING = ON
);





-- albo od razu wczamy wersjonowanie przy tworzeniu tabeli
CREATE TABLE dbo.Product10 (
	ID				INT IDENTITY CONSTRAINT PK_Product10 PRIMARY KEY,
	Name			VARCHAR(15) NOT NULL,
	Price			SMALLMONEY,

	PERIOD FOR SYSTEM_TIME (ValidFrom, ValidTo),

	ValidFrom		DATETIME2(0) GENERATED ALWAYS AS ROW START,
	ValidTo			DATETIME2(0) GENERATED ALWAYS AS ROW END
) WITH (
	SYSTEM_VERSIONING = ON
)


-- Opcja 2) Modyfikujemy struktur istniejcej tabeli dodajc definicj okresu
ALTER TABLE dbo.Product ADD
	PERIOD FOR SYSTEM_TIME (ValidFrom, ValidTo),

	ValidFrom		DATETIME2(0) GENERATED ALWAYS AS ROW START
		CONSTRAINT DF_Product_ValidFrom DEFAULT '1900-01-01',

	ValidTo			DATETIME2(0) GENERATED ALWAYS AS ROW END
		CONSTRAINT DF_Product_ValidTo DEFAULT '9999-12-31 23:59:59'
;
GO

-- wczamy wersjonowanie; oprcz tego moemy napisa
-- jak ma si nazywa tabela historyczna (moe by w innym schemacie, 
-- ale musi by w tej samej bazie);
-- w ten sam sposb moemy podczy ju istniejca tabel z histori!

ALTER TABLE dbo.Product
SET (
	SYSTEM_VERSIONING = ON (HISTORY_TABLE = hist.Product)
);


-- kiedy odpytamy tabel dbo.Product, to widzimy domylnie kolumny okresu wanoci
-- ale moemy je ukry przed SELECT *:
SELECT * FROM dbo.Product

CREATE TABLE dbo.Product20 (
	ID				INT IDENTITY CONSTRAINT PK_Product20 PRIMARY KEY,
	Name			VARCHAR(15) NOT NULL,
	Price			SMALLMONEY,

	PERIOD FOR SYSTEM_TIME (ValidFrom, ValidTo),

	ValidFrom		DATETIME2(0) GENERATED ALWAYS AS ROW START	HIDDEN,
	ValidTo			DATETIME2(0) GENERATED ALWAYS AS ROW END	HIDDEN
) WITH (
	SYSTEM_VERSIONING = ON (HISTORY_TABLE = hist.Product20)
)

SELECT * FROM dbo.Product20;
SELECT *, ValidFrom, ValidTo FROM dbo.Product20;

ALTER TABLE dbo.Product20
SET (
	SYSTEM_VERSIONING = OFF
);

DROP TABLE dbo.Product20;
DROP TABLE hist.Product20;
GO

-- poniewa dodajemy kolumny do istniejcej tabeli, mog istnie w niej ju dane
-- dlatego musimy doda do kolumn ValidFrom, ValidTo definicje wartoci domylnych
-- inaczej otrzymamy bd:

-- Msg 4901, Level 16, State 1, Line 83
-- ALTER TABLE only allows columns to be added that can contain nulls, 
-- or have a DEFAULT definition specified, or the column being added is 
-- an identity or timestamp column, or alternatively if none of the previous conditions 
-- are satisfied the table must be empty to allow addition of this column. 
-- Column 'ValidFrom' cannot be added to non-empty table 'Product' because 
-- it does not satisfy these conditions.

-- warto domylna dla kolumny ValidTo musi by najwiksz wartoci datetime2 
-- dla danej precyzji (np. 9999-12-31 23:59:59.99 dla datetime(2)
-- inaczej otrzymamy bd:

-- Msg 13575, Level 16, State 0, Line 83
-- ADD PERIOD FOR SYSTEM_TIME failed because table 'SQLDayLite2015.dbo.Product' 
-- contains records where end of period is not equal to MAX datetime.


-- do przetestowania przy Stretch Database
USE SQLDayLite2015;
GO
CREATE TABLE dbo.ProductStretch (
	ID				INT IDENTITY CONSTRAINT PK_Product10 PRIMARY KEY,
	Name			VARCHAR(15) NOT NULL,
	Price			SMALLMONEY,

	PERIOD FOR SYSTEM_TIME (ValidFrom, ValidTo),

	ValidFrom		DATETIME2(0) GENERATED ALWAYS AS ROW START,
	ValidTo			DATETIME2(0) GENERATED ALWAYS AS ROW END
) WITH (
	SYSTEM_VERSIONING = ON (HISTORY_TABLE = hist.ProductStretchH)
)

ALTER TABLE dbo.ProductStretch SET (SYSTEM_VERSIONING = OFF)
GO

ALTER TABLE dbo.ProductStretch SET (SYSTEM_VERSIONING = ON )
GO

DROP TABLE dbo.ProductStretch
GO

DROP TABLE hist.ProductStretch
GO